

#include <iostream>
#include <clocale>
#include <cstdlib>

#include <string>
#include <fstream>
//#include <codecvt>

#include <vector>
#include <sys/types.h>
#include <sys/time.h>

#include "passport_visa.h"
#include "tdrocr.h"
#include "common.h"

#ifdef USE_ISSUER_ADDR 
using namespace std;
vector<string> item_list;
HASH_TABLE_STRUCT hash_tbl_struct;
//  AP Hash Function 
unsigned int APHash( char * str)
{
    unsigned  int  hash  =   0 ;
     int  i;
 
     for  (i = 0 ;  * str; i ++ )
    {
         if  ((i  &   1 )  ==   0 )
        {
            hash  ^=  ((hash  <<   7 )  ^  ( * str ++ )  ^  (hash  >>   3 ));
        }
         else 
        {
            hash  ^=  ( ~ ((hash  <<   11 )  ^  ( * str ++ )  ^  (hash  >>   5 )));
        }
    }
 
     return  (hash  &   0x7FFFFFFF );
}


int set_hash_tbl(char *item_name, HASH_TABLE_STRUCT *hash_tbl_str) 
{
  unsigned int value;
  char *key = (char *)item_name;
  if(!*key)  
    return ERROR_CITY_LIST_INPUT;

  if(hash_tbl_str->hash_tbl_idx >= Hash_Table_Max_Size -1)
    return ERROR_CITY_LIST_HASH_TYPE;
  
  value = APHash(key);    
  hash_tbl_str->hash_table[hash_tbl_str->hash_tbl_idx++] = value;      
  // printf("key = %s. value = %x .\n", key, value);  

  return 0;
} 

int get_frist_char(char* s, char *first_byte){
  int type ;//= s[0]&0xF0; 
  int cnt = strlen(s);
  int char_len = 0;
  int target_char_pos = 0;

  // 判断至少包含2个汉字字符
  for(int i = 0; i < 2; i++){
    type = (s[target_char_pos] & 0xF0);
    
    switch(type) {
      case 0xC0:
      char_len = 2;
      break;
      case 0xE0:
      char_len = 3; 
      break;
      case 0xF0:
      char_len = 4;
      break;
      default:
      
      return ERROR_CITY_LIST_INPUT;
    }
  
    target_char_pos += char_len;
    if(cnt < target_char_pos) {
      
      return ERROR_CITY_LIST_INPUT;
    }
  }

  //memcpy(first_byte, s, target_char_pos);
  return 0;
}

int check_cityList (char *item_name, HASH_TABLE_STRUCT *hash_tbl_str, int* pos_idx) 
{
  unsigned int value, left,right,mid, store_value;
  char *key = ( char *)item_name;
  
  if(!*key)  
    return ERROR_CITY_LIST_INPUT;

  //char first_byte[10] = {0};
  // 当前版本仅校验 输入数据是否包含一个合法汉字
  //int rsp = get_frist_char(key, first_byte);
 
#if 1
  //if(0 != rsp)
    //return rsp;

  if(hash_tbl_str->hash_tbl_idx <= 0)
    return ERROR_CITY_LIST_HASH_TYPE;
    
  //value = APHash(first_byte);

  value = APHash(key);
  //printf(" value = %x .\n", value);  
  left = 0;
  right = hash_tbl_str->hash_tbl_idx ;
  while (left <= right ) {
    mid = (right + left)/2;
    store_value = hash_tbl_str->hash_table[mid];
    if(store_value < value) {
      left = mid +1;
    }
    else if(store_value > value) {
      right = mid -1;
      if(0 == mid)
         return  ERROR_CITY_LIST_NO_EXIST_VALUE;
    }
    else {
      *pos_idx = mid;
      //printf("find idx= %d. value = %x .\n", mid, value);  
      break;
    }
  }
    
  if(left > right)
    return  ERROR_CITY_LIST_NO_EXIST_VALUE;
      
  return 0; 
#else
  return rsp;
#endif 
}


//int create_cityList_map()
int load_city_dict(char * dir)
{
  #if 1
  //std::fstream fin("cityList_1.js", std::ios::in | std::ios::binary);
  std::fstream fin(dir, std::ios::in | std::ios::binary);
  if(!fin) {
    LOGI("open in file [%s] error", dir);
    return ERROR_CITY_LIST_INPUT;
  }

  char s[100];

  while(fin.getline((char *)&s[0],100)) { //一直读到文件结束
    #if 1
    item_list.push_back(s);
    #else
    // 考虑到长汉字的缩写规则不定，暂时只判断首字节
    char first_byte[10] = {0};
    get_frist_char(s, first_byte);
    item_list.push_back(first_byte);
    #endif
  }
  
  int item_cnt = item_list.size();
  LOGI("city_dict_count=%d", item_cnt);

  if(item_cnt >= Hash_Table_Max_Size) {
    //printf("beyond hash table  .\n");
    return ERROR_CITY_LIST_NO_EXIST_POS ;
  }
 
  char tmp[100];
  int rsp ;

    //double start=tdr_get_current_time(); 
    memset((char*)&hash_tbl_struct, DEFAULT_VALUE, sizeof(HASH_TABLE_STRUCT));
    hash_tbl_struct.hash_tbl_idx = 0;
    
    for(int n = 0; n < item_cnt; n++) {
      strcpy(tmp, item_list.at(n).c_str());
      rsp = set_hash_tbl(tmp, &hash_tbl_struct);
      if(0 != rsp) {
        LOGI("set_hash_tbl error %d . %s", rsp, tmp);
        return ERROR_CITY_LIST_NO_EXIST_VALUE;
      }
    }
    
    sort(&hash_tbl_struct.hash_table[0], &hash_tbl_struct.hash_table[hash_tbl_struct.hash_tbl_idx]);
   
    //double end=tdr_get_current_time(); 
    //printf("create_cityList_h_file cost %f ms .\n", end-start);

#endif

  return 0;
}

#endif