#include <opencv2/opencv.hpp>
#include <iostream>
#include "common.h"
#include <omp.h>
#include "tdrocr.h"
#include "utils.h"
#include "passport_visa.h"

extern int g_check_edge;
extern vector<RotatedRect>  mrz_rotatedRect;
int isOldCamera();
cv::Mat getMserRegion(Mat& src, int color_mode);
myTextRect detectLettersRect(Mat& src, int morph_size); // default morph_size=20
int char_segment(int iscard, cv::Mat &line1, cv::Mat &line2, cv::Mat &line1_rgb, cv::Mat &line2_rgb, int &sz1, int &sz2);
void BGR2GRAY(cv::Mat & input, cv::Mat & output, int color_mode);
bool rotation_visa2(Mat &in, Mat &out, Size rect_size, Point2f center, const double angle, PASSPORT_ROTATED_STRUCT &rotated_struct, bool set_rotated_struct);
PASSPORT_ROTATED_STRUCT g_rotated_struct[2];
int charDtct(Mat& src,Mat & maserMask,  Mat& line1_rgb, Mat& line2_rgb, Mat& line1, Mat& line2, int &sz1, int &sz2);
void detect_brightness(cv::Mat input_img, float& cast, float& da);
int  getBinThreshold(cv::Mat & src);
//int is_outBound(Mat &src, Mat &roi, Size rect_size, Point2f center, float angle, Point2f ori_center, bool is_line1, int th = 9, int offset_th = 3);
bool rotation_visa(Mat& in, Mat& out, Size rect_size, Point2f center, const double angle, 
      PASSPORT_ROTATED_STRUCT &rotated_struct, bool set_rotated_struct) {//set_rotated_struct 用于核并行保护
        return(rotation_visa2(in, out, rect_size, center, angle, rotated_struct, set_rotated_struct));
}

void addWhiteEdge(vector<Rect>& posRect,  cv::Mat& dst)
{
        vector<Rect>::iterator  itr;
        for (itr = posRect.begin()
                        ; itr != posRect.end()
                        ; itr ++)
        {

                cv::rectangle(dst,cvPoint(itr->x,itr->y),cvPoint(itr->x+itr->width-1, itr->y+itr->height-1),Scalar(255,0,0));
                cv::rectangle(dst,cvPoint(itr->x+1,itr->y+1),cvPoint(itr->x+itr->width-2, itr->y+itr->height-2),Scalar(255,0,0));
        }
}


vector<int> filterLightline(Mat & img)
{
    Mat img_gray, img_sobel, img_threshold, element;
    if(img.channels()>1)    cvtColor(img, img_gray, CV_BGR2GRAY);
    else img_gray = img;

    Mat img_gray_t;
    cv::adaptiveThreshold(img_gray, img_gray_t, 255, CV_ADAPTIVE_THRESH_MEAN_C, CV_THRESH_BINARY, 81, 33);//81,33
    element = getStructuringElement(cv::MORPH_RECT, Size(10, 10));//7,1 9,2
    morphologyEx(img_gray_t, img_gray_t, CV_MOP_CLOSE, element); //Does the trick
    cv::copyMakeBorder(img_gray_t,img_gray_t,0,0,1,0,BORDER_CONSTANT,Scalar(0)); //www
    element = getStructuringElement(cv::MORPH_RECT, Size(8, 8));
    erode(img_gray_t,img_gray_t,element);
    Sobel(img_gray, img_sobel, CV_8U, 1, 0, 3, 1, 0, BORDER_DEFAULT);
    Rect roi_t(1,0,img.cols,img.rows); //www
    img_gray_t = img_gray_t(roi_t);
    Mat sobel_new = Mat::zeros(img.rows,img.cols,CV_8UC1);
    img_sobel.copyTo(sobel_new,img_gray_t);
    threshold(sobel_new, img_threshold, 0, 255, CV_THRESH_OTSU+CV_THRESH_BINARY);
    vector<int> lightline(2);
    int w_min = img_threshold.cols;
    int w_max = 0;
    vector< vector< Point> > contours_no_lightline;
    findContours(img_threshold, contours_no_lightline, 0, 1);
    vector<vector<Point> > contours_poly_no_lightline( contours_no_lightline.size() );
    for( size_t i = 0; i < contours_no_lightline.size(); i++){
        approxPolyDP( Mat(contours_no_lightline[i]), contours_poly_no_lightline[i], 0, true );
        Rect appRect_no_lightline( boundingRect( Mat(contours_poly_no_lightline[i]) ));
        if(appRect_no_lightline.height>50 && appRect_no_lightline.width<15){
            w_min = appRect_no_lightline.x+appRect_no_lightline.width < w_min? appRect_no_lightline.x+appRect_no_lightline.width : w_min;
            w_max = appRect_no_lightline.x > w_max? appRect_no_lightline.x : w_max;
        }
    }
    lightline[0] = w_min+4;
    lightline[1] = w_max-4;
    return lightline;
}


bool comp_Point2f_x(Point2f a, Point2f b)
{
    return(a.x < b.x);
}


int checkLettersRect(Mat & src, Mat & resizeimg, myTextRect & textRect, Mat & line1, Mat & line2, Mat & line1_gray, Mat & line2_gray,float scale, int width, int color_mode)
{

    if(textRect.rotatedRect.size()==0)
    {
        showError(ERROR_OTR_NONE);
        return ERROR_OTR_NONE;
    }
    if(textRect.rotatedRect.size()==1)
    {
        int rw = max(textRect.rotatedRect[0].size.width,textRect.rotatedRect[0].size.height);
        int rh = min(textRect.rotatedRect[0].size.width,textRect.rotatedRect[0].size.height);
        int thres_1=30;
        int thres_2 = 10; //www
        //if (rh<=thres_1 && rh>thres_2 && rw<(width-20) && rw>width*0.5)
        if (rh<=thres_1 && rh>thres_2 && rw<(width-20) && rw>500)
        {
               showError(ERROR_OTR_ONE);
  	       return ERROR_OTR_ONE;
        }
        else
        {
            showError(ERROR_OTR_NONE);
            return ERROR_OTR_NONE;
        }
    }
//printf("textRectsize=%d\n", textRect.rotatedRect.size());

    sort(textRect.rotatedRect.begin(),textRect.rotatedRect.end(),comp);

    //int thres=36; //33, 30
    int thres=56;//30; //33, 30
    int thres1 =10;

    int rw1, rh1, rw2, rh2;


    int found = 0;

    rw1 = max(textRect.rotatedRect[0].size.width,textRect.rotatedRect[0].size.height);
    rh1 = min(textRect.rotatedRect[0].size.width,textRect.rotatedRect[0].size.height);
    rw2 = max(textRect.rotatedRect[1].size.width,textRect.rotatedRect[1].size.height);
    rh2 = min(textRect.rotatedRect[1].size.width,textRect.rotatedRect[1].size.height);

    //if(rh1<=thres && rh2<=thres && rh1>thres1 && rh2>thres1 && rw1<(width-20) && rw2<(width-20) && rw1>width*0.5 && rw2>width*0.5 && abs(rw1-rw2)<20) found = 1;
    //if(rh1<=thres && rh2<=thres && rh1>thres1 && rh2>thres1 && rw1<(width-20) && rw2<(width-20) && rw1>500 && rw2>500 && abs(rw1-rw2)<20) found = 1;
    //if(rh1<=thres && rh2<=thres && rh1>thres1 && rh2>thres1 && rw1<(width-20) && rw2<(width-20) && rw1>500 && rw2>500 && abs(rw1-rw2)<20){
    if(rh1<=thres && rh2<=thres && rh1>thres1 && rh2>thres1 && rw1<(width-20) && rw2<(width-20) && rw1>440 && rw2>440 && abs(rw1-rw2)<30){
         found = 1;
    }
    else{
        if(textRect.rotatedRect.size()>2){
            int rh_tmp[2];
            RotatedRect roi_tmp[3];
            Point2f pts_0[4], pts_1[4], pts_2[4];
            vector<Point> roi_counts;
            vector<Point2f> point_set;
            for(unsigned int i = 1; i < textRect.rotatedRect.size()-1; i++){
                for(unsigned int j = textRect.rotatedRect.size()-1; j > i; j--){
                    roi_tmp[0] = textRect.rotatedRect[i];
                    roi_tmp[1] = textRect.rotatedRect[j];
                    rh_tmp[0] = min(roi_tmp[0].size.width,roi_tmp[0].size.height);
                    rh_tmp[1] = min(roi_tmp[1].size.width,roi_tmp[1].size.height);
                    if(rh_tmp[0]<=thres && rh_tmp[1]<=thres &&  rh_tmp[0]>thres1 && rh_tmp[1]>thres1){
                          roi_tmp[0].points(pts_0);
                          roi_tmp[1].points(pts_1);
                          for(unsigned int i = 0; i < 4; i++){
                              point_set.push_back(pts_0[i]);
                              point_set.push_back(pts_1[i]);
                          }
                          sort(point_set.begin(), point_set.end(), comp_Point2f_x);
                          pts_2[0] = point_set[0];
                          pts_2[1] = point_set[1];
                          pts_2[2] = point_set[6];
                          pts_2[3] = point_set[7];
                          for(unsigned int i = 0; i < 4; i++) roi_counts.push_back(pts_2[i]);
                          roi_tmp[2] = minAreaRect(roi_counts);
                          vector<Point2f>().swap(point_set);
                          vector<Point>().swap(roi_counts);
                    }
                    rw2 = max(roi_tmp[2].size.width,roi_tmp[2].size.height);
                    rh2 = min(roi_tmp[2].size.width,roi_tmp[2].size.height);
                    //if((abs(roi_tmp[2].angle) < 3 || abs(roi_tmp[2].angle) > 87) && rh1<=thres && rh2<=thres &&
                          //rh1>7 && rh2>7 && rw1<(width-20) && rw2<(width-20) && rw1>width*0.5 && rw2>width*0.5 && abs(rw1-rw2)<20)
                    if((abs(roi_tmp[2].angle) < 3 || abs(roi_tmp[2].angle) > 87) && rh1<=thres && rh2<=thres && 
                          rh1>7 && rh2>7 && rw1<(width-20) && rw2<(width-20) && rw1>500 && rw2>500 && abs(rw1-rw2)<20)

                    {
                          textRect.rotatedRect[1]=roi_tmp[2];
                          int h_diff = abs(textRect.rotatedRect[0].center.y-textRect.rotatedRect[1].center.y);
                          if (h_diff<40) 
                          { 
                                found = 1;
                          }
                            //cout<<"angle: "<<abs(roi_tmp[2].angle)<<endl;
                    }
                    if(found == 1) break;
                }
                if(found == 1) break;
            }
        }
    }

    if(found!=1){
        //if(rh1<=thres && rh1>thres1 && rw1<(width-20) && rw1>width*0.5)
        if(rh1<=thres && rh1>thres1 && rw1<(width-20) && rw1>500)
        {
               showError(ERROR_OTR_ONE);
  	       return ERROR_OTR_ONE;
        }
        else
        {
            showError(ERROR_OTR_NONE);
            return ERROR_OTR_NONE;
        }

    }

#ifdef TDR_TRACE
        //LOGD("width=%d,rw1/rh1/rw2/rh2=%d/%d/%d/%d", width,rw1,rh1,rw2,rh2);
#endif

	int lt1,lt2;

	Rect_<float> safeBoundRct0[2];
	for(int i=0;i<2;i++)
	{
		RotatedRect roi_rect = textRect.rotatedRect[i];
		bool isFormRect = calcSafeRect(roi_rect,resizeimg,safeBoundRct0[i]);
		if(!isFormRect)
		{
			showError(ERROR_SAFE_RECT);
			return ERROR_SAFE_RECT;
		}
	}
  
	lt1 = safeBoundRct0[0].y;
	lt2 = safeBoundRct0[1].y;

	Rect_<float> safeBoundRct[2];
	RotatedRect roi_rect[2]; 
        if(lt1<lt2) {
              safeBoundRct[0]=safeBoundRct0[0];
              safeBoundRct[1]=safeBoundRct0[1];
              roi_rect[0]=textRect.rotatedRect[0];
              roi_rect[1]=textRect.rotatedRect[1];
        }
        else {
              safeBoundRct[0]=safeBoundRct0[1];
              safeBoundRct[1]=safeBoundRct0[0];
              roi_rect[0]=textRect.rotatedRect[1];
              roi_rect[1]=textRect.rotatedRect[0];
        }

        RotatedRect r_rect;
        if(roi_rect[0].center.y < roi_rect[1].center.y) r_rect = roi_rect[0];
        else r_rect = roi_rect[1]; 
        r_rect.center.x*=scale;
        r_rect.center.y*=scale;

        RotatedRect rr;
        if(mrz_rotatedRect.size()>0) mrz_rotatedRect.clear();    // 存放两行mrz的旋转外接矩形，在最后裁切证件图像时定位用
        for(int j=0; j<2; j++){
          rr = roi_rect[j];
          rr.center.x *= scale;
          rr.center.y *= scale;
          rr.size.height *= scale;
          rr.size.width *= scale;
          mrz_rotatedRect.push_back(rr);
        }

	Point2f roi_rect_center[2];
        Point2f roi_rect_c[2];
	float roi_angle[2];
	Rect_<float> srcrect[2];
	Mat bm[2];
        Mat bm_gray[2];
	Size roi_rect_size[2];
	Size rrs[2];
	Point2f rrc[2];
	bool  wrong = false;
        int  char_wrong = 0;

         Point st(src.cols-1,src.rows-1);
         Point tl;
	{
#pragma omp parallel for num_threads(2)
		for(int i=0;i<2;i++)
		{
      roi_rect_c[i] = roi_rect[i].center*scale;
			roi_rect_center[i] = roi_rect[i].center - safeBoundRct[i].tl();
			roi_angle[i] = roi_rect[i].angle;
			if(roi_angle[i] < -45) roi_angle[i] += 90;

			srcrect[i].x = safeBoundRct[i].x*scale;
			srcrect[i].y = safeBoundRct[i].y*scale;
			srcrect[i].width = safeBoundRct[i].width*scale;
			srcrect[i].height = safeBoundRct[i].height*scale;

			bm[i] = src(srcrect[i]);

			roi_rect_size[i].width = max(roi_rect[i].size.width,roi_rect[i].size.height)+2;//8; //5
			roi_rect_size[i].height = min(roi_rect[i].size.width,roi_rect[i].size.height)+6;//+2;//5; //5

			rrs[i].width = roi_rect_size[i].width*scale;
			rrs[i].height = roi_rect_size[i].height*scale;

			rrc[i].x = roi_rect_center[i].x*scale;
			rrc[i].y = roi_rect_center[i].y*scale;

			if(rrc[i].x+rrs[i].width/2>src.cols
					||rrc[i].y-rrs[i].height/2<0
					||rrc[i].y+rrs[i].height/2>src.rows)
			{ 
				wrong = true;
			} 

                        if(st.y > srcrect[i].y)
                        {
                            st.x = srcrect[i].x;
                            st.y = srcrect[i].y;
                        }

                        bool bret=false;
                        if(i==0){
                             bret=rotation_visa(bm[i],line1,rrs[i],rrc[i],roi_angle[i],g_rotated_struct[i],true);    
                             BGR2GRAY(line1, line1_gray, color_mode);
                        }
                        else{
                             bret=rotation_visa(bm[i],line2,rrs[i],rrc[i],roi_angle[i],g_rotated_struct[i],false);    
                             BGR2GRAY(line2, line2_gray, color_mode);
                        }

                        g_rotated_struct[i].rrect = srcrect[i];                              

                        if(bret==false) {
                               wrong=true;
                               //break;
                        }
		} 
	} 

	if (wrong)
        {
            showError(ERROR_ROTATION);
            return ERROR_ROTATION;
        }

  if (isOldCamera()==1 && g_check_edge==1 && char_wrong!=0){
    return char_wrong;
  }

        int first=0;
        if(srcrect[0].y < srcrect[1].y) first=0;
        else first=1;

        tl.x = rrc[first].x-rrs[first].width/2+st.x;
        tl.y = rrc[first].y-rrs[first].height/2+st.y;
        

	return 0;
}

#if 0
int charDtct(Mat& src, Mat& line1_rgb, Mat& line2_rgb, Mat& line1, Mat& line2, int &sz1, int &sz2)
{
      int width, height;
      Mat resizeimg;
      if(src.cols < 600) {
            showError(ERROR_IMAGE_SIZE);
            return ERROR_IMAGE_SIZE;
      }
      int  scale=src.cols/600;
      width = src.cols/scale;
      height = src.rows/scale;
      if( src.cols<=0 || src.rows<=0 || width<=0 || height<=0 ) return -1;
      resize(src,resizeimg,Size(width,height));
      cv::Mat mserMask;
      int ret = -1;
      int color_mode=0;
      LOGD("color_mode:%d, threshold:%d,%d", color_mode, threshold1, threshold2);

      mserMask=getMserRegion(src, color_mode);

#ifdef TDR_TRACE
if(isDebugShow())
{
char title[20];
sprintf(title, "mser");
imshow(title, mserMask);
}
#endif

     return (_charDtct(src, resizeimg, mserMask,  line1_rgb, line2_rgb, line1, line2, sz1, sz2));

}
#endif


int charDtct(Mat& src, Mat & mserMask,  Mat& line1_rgb, Mat& line2_rgb, Mat& line1, Mat& line2, int &sz1, int &sz2)
{

      int step=0;
      int ret;
      float scale=2;//(float)src.cols/680;
      int width = (float)src.cols/scale;
      int height = (float)src.rows/scale;
      cv::Mat line1_gray, line2_gray, resizeimg;
      int color_mode=0;
      int threshold1=31;
      int threshold2=17;
      if(g_detect_mode==0) threshold2=11;

      if(src.cols < 600) {
            showError(ERROR_IMAGE_SIZE);
            return ERROR_IMAGE_SIZE;
      }

      if( src.cols<=0 || src.rows<=0 || width<=0 || height<=0 ) return -1;
      resize(src,resizeimg,Size(width,height));

      LOGD("detect letter: 6");
      myTextRect textRect = detectLettersRect(mserMask, 6); //12
      ret = checkLettersRect(src,resizeimg,textRect,line1_rgb, line2_rgb,line1_gray, line2_gray,scale,width, color_mode);
      if(ret==ERROR_OTR_ONE|| ret==ERROR_OTR_NONE)
      {
               step=1;
#if 1
               LOGD("detect letter: 15"); 
               textRect = detectLettersRect(mserMask, 15);
               ret = checkLettersRect(src,resizeimg,textRect,line1_rgb, line2_rgb,line1_gray, line2_gray,scale,width, color_mode);
               if(ret==ERROR_OTR_ONE|| ret==ERROR_OTR_NONE)
#endif
               {
                    LOGD("detect letter: 20"); //20
                    textRect = detectLettersRect(mserMask, 20);
                    ret = checkLettersRect(src,resizeimg,textRect,line1_rgb, line2_rgb,line1_gray, line2_gray,scale,width, color_mode);
               }
      }

      if (ret == 0){
           LOGD("try threshold %d", threshold2);
           preprocess1(line1_gray, line2_gray, line1,line2, threshold1, threshold2);
           ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);
           if(ret!=0 && step!=1)
           {
               LOGD("detect letter: 15");
               textRect = detectLettersRect(mserMask, 15);
               ret = checkLettersRect(src,resizeimg,textRect,line1_rgb, line2_rgb,line1_gray, line2_gray,scale,width, color_mode);          
               if(ret==0){
#if 1
                  if(sz1>44 || sz2>44)
                  {
                     //LOGD("try threshold 25");
                     //preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 25);
                     //ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);

                     LOGD("try threshold 17");
                     preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 17);
                     ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);
                  }
                  else{
                     LOGD("try threshold 17");
                     preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 17);
                     ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);

                     //LOGD("try threshold 25");
                     //preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 25);
                     //ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);
                  }
#endif
                     //if(threshold2==11) threshold2=17;
					 //else if(threshold2==17) threshold2=11;
                     //LOGD("try threshold %d", threshold2);
                     //preprocess1(line1_gray, line2_gray, line1,line2, threshold1, threshold2);
                     //ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);
               }
           }
#if 1
           else if(ret!=0 && step==1)
           {
                  if(sz1>44 || sz2>44){
                     //LOGD("try threshold 25");
                     //preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 25);
                     //ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);

                     LOGD("try threshold 17");
                     preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 17);
                     ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);
                  }
                  else{
                     LOGD("try threshold 17");
                     preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 17);
                     ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);

                     //LOGD("try threshold 25");
                     //preprocess1(line1_gray, line2_gray, line1,line2, threshold1, 25);
                     //ret=char_segment(0, line1, line2,line1_rgb, line2_rgb,  sz1, sz2);
                  }
           }
#endif
      }


      return ret;
}


cv::Mat getMserRegion(Mat& src, int color_mode)
{
  int width, height;
  Mat img,gray;
  float scale_800=src.cols/600;
  width = src.cols/scale_800;
  height = src.rows/scale_800;

  int widthh = width*0.75;
  int heighth = height*0.75;

  resize(src, img, Size(widthh,heighth),cv::INTER_NEAREST);
  if(img.channels()>1)
  {
    BGR2GRAY(img, gray, color_mode);

#ifdef TDR_TRACE
if(isDebugShow())
{
    cv::imshow("gray", gray);
}
#endif

  }
  else
  {
    gray = img;
  }

  cv::GaussianBlur(gray,gray,cv::Size(3,3),1,0);

  cv::Ptr<cv::MSER> ptrMSER = cv::MSER::create(5,8,100,0.5);  //(5,20,150,0.3)  (7,10,100,0.45) 5,8,100,0.5

  std::vector<std::vector<cv::Point> > points;
  std::vector<cv::Rect> rects;
  ptrMSER->detectRegions(gray,points,rects);

  Mat test_mask = Mat::zeros(img.rows,img.cols,CV_8UC1);

  vector<cv::Rect> charRects;
  for(int i=0;i < points.size();i++)
  {
    Rect brect = boundingRect(points[i]);
    int width = brect.width;
    int height = brect.height;
    if (float(height)/float(width) < 4 && float(height)/float(width) >0.25)
    {
      charRects.push_back(brect);
    }
  }

  for (int i=0;i < charRects.size();i++) {
    rectangle(test_mask, charRects[i], Scalar(255), -1);
    //rectangle(test_mask, charRects[i], Scalar(255), 1);
  }
#ifdef TDR_TRACE
//imshow("mser src", test_mask);
#endif

  resize(test_mask, test_mask, Size(width,height),cv::INTER_NEAREST);
  return test_mask;
}

//cv::Ptr<cv::MSER> ptrMSER = cv::MSER::create(2,8,60,0.6);//(5,8,100,0.5);  //(5,20,150,0.3)  (7,10,100,0.45) 5,8,100,0.5


myTextRect detectLettersRect(Mat& src, int morph_size) // default morph_size=20
{
  Mat element = getStructuringElement(MORPH_RECT, Size(morph_size, 1) );
  Mat linemask= Mat::zeros(src.rows,src.cols,CV_8UC1);
  morphologyEx(src, linemask, CV_MOP_CLOSE, element);
#ifdef TDR_TRACE
if(isDebugShow()) 
{
    //imshow("mser", src);
    //imshow("mser_closexxx", linemask);
}
#endif

  vector<vector<Point> > contours_mask;
  findContours(linemask, contours_mask, 0,1);

  myTextRect textRect;
  vector<vector<Point> > contours_poly( contours_mask.size() );
  for( size_t i = 0; i < contours_mask.size(); i++ ) {
    if (contours_mask[i].size()>100)
    {
      approxPolyDP( Mat(contours_mask[i]), contours_poly[i], 0, true );
      Rect appRect( boundingRect( Mat(contours_poly[i]) ));
      if (appRect.width>appRect.height)
      {
        textRect.boundRect.push_back(appRect);
        textRect.rotatedRect.push_back(minAreaRect(contours_poly[i]));
      }
    }
  }

  return textRect;
}

