#include <opencv2/opencv.hpp>
#include <iostream>
#include "common.h"
#include <omp.h>
#include "tdrocr.h"
#include "utils.h"
#include "passport_visa.h"
#include "tdr_crnn.h"
#include "result.h"

bool rotation_visa2(Mat &in, Mat &out, Size rect_size, Point2f center, const double angle, PASSPORT_ROTATED_STRUCT &rotated_struct, bool set_rotated_struct);
int detectMrz(Mat& src, vector<RotatedRect> & rr, int isRefined);
extern PASSPORT_ROTATED_STRUCT g_rotated_struct[2];
int charDtct3(Mat& src, Mat& line1_rgb, Mat& line2_rgb, Mat& line1_bin, Mat& line2_bin, int &sz1, int &sz2);
int passportMatchOcr3(int sz1, int sz2, OCR_RESULT &ocr_result);
extern int check_mrz_main(Mat& src, Mat &masrMask);
extern vector<RotatedRect>  mrz_rotatedRect;
int isRotateRectOutline(Mat & src, vector<RotatedRect> & rr);

int passport_location3(cv::Mat& src, int &sz1, int &sz2, cv::Mat & line1_rgb, cv::Mat & line2_rgb)
{
        double start=tdr_get_current_time();
        clearVisaIssueInfo();

        CUT_SIZE=424;//480;

        Mat roi = src(Rect(0, src.rows-CUT_SIZE-1,  src.cols, CUT_SIZE));
        cv::Mat line1, line2;
        int ret;

        //////////////////////////////////
        cv::Mat mserMask;
        ret =  check_mrz_main(roi, mserMask);
        LOGD("mrz check: %d", ret);
        if(ret != 0)
            return ret;
        /////////////////////////////////

LOGD("---------------  RETRY       -------------");
        ret = charDtct3(roi, line1_rgb, line2_rgb, line1, line2, sz1, sz2);

        double end=tdr_get_current_time();
        LOGD("locat and split char cost %.3f ms", end-start);
        return ret;
}

int charDtct3(Mat& src, Mat& line1_rgb, Mat& line2_rgb, Mat& line1_bin, Mat& line2_bin, int &sz1, int &sz2)
{
      int scale, width, height;
      Mat resizeimg;
      Mat line1_gray, line2_gray;

      if(src.cols < 600) {
            showError(ERROR_IMAGE_SIZE);
            return ERROR_IMAGE_SIZE;
      }

      scale=src.cols/600;
      float scale_x= scale;
      float scale_y= scale;
      width = src.cols/scale_x;
      height = src.rows/scale_y;
      if( src.cols<=0 || src.rows<=0 || width<=0 || height<=0 ) return -1;
      vector<RotatedRect>  rr;
      int ret=detectMrz(src, rr, 1);
      if(ret<0) return -1;

      ret=isRotateRectOutline(src, rr);
      if(ret<0){
          if(ret==-1) {
              LOGD("outline: bottom");
              return ERROR_CHAR_BOTTOM_IMG;
          }
          else if(ret==-2){
              LOGD("outline: top");
              return ERROR_CHAR_TOP_IMG;
          }
          return -1;
      }

      //if(isMoved(0, rr[0].center.x, rr[0].center.y)) return ERROR_NEXT;

      bool rets[2]; rets[0]=false; rets[1]=false;

      cv::Mat small[2];
      Point2f new_center[2];
      Rect2f safe_rect[2];
      for(int i=0; i<2; i++)
      {
            if (rr[i].angle < -45) rr[i].angle += 90;
            if(rr[i].size.height>rr[i].size.width){
                swap(rr[i].size.height, rr[i].size.width);
            }
            bool isFormRect = calcSafeRect(rr[i], src, safe_rect[i]);
            small[i] = src(safe_rect[i]);
            new_center[i] = rr[i].center - cv::Point2f(safe_rect[i].x, safe_rect[i].y);
      }
      if(mrz_rotatedRect.size()>0) mrz_rotatedRect.clear();   // 存放两行mrz的旋转外接矩形，在最后裁切证件图像时定位用
      mrz_rotatedRect = rr;

//#ifndef TDR_TRACE
#pragma omp parallel for num_threads(2)
//#endif
      for(int i=0; i<2; i++)
      {
        if (i == 0){
            rets[i] = rotation_visa2(small[i], line1_rgb, rr[i].size, new_center[i], rr[i].angle, g_rotated_struct[i], true);
            g_rotated_struct[i].rrect = safe_rect[i];
        }
        else
            rets[i] = rotation_visa2(small[i], line2_rgb, rr[i].size, new_center[i], rr[i].angle, g_rotated_struct[i], false);

      }

      if(!rets[0] || !rets[1]) return -1;
      return 0;
}

